@@ -34,8 +34,9 @@ logger = logging.getLogger(__name__)
 
 import time
 sparql_midnight_dateTime_format = "%Y-%m-%dT00:00:00Z"
-SECONDS_PER_DAY = 86400
 
+SECONDS_PER_DAY = 86400
+PUNCTUATION_FILTER = " !\\\"#$%&'()*+,-./:;<=>?@[\\\\]^_`{|}~"
 
 class Query():
 
@@ -64,23 +65,40 @@ class Query():
         return '<Query>'
 
     @staticmethod
-    def order_by_statement(attr):
-        """Returns a SPARQL ORDER BY statement sorting by the given attribute, ignoring
-            articles as defined in _("the"). 'Attr' should be given without parentheses,
-            e.g., "attr='?author'"."""
-        return_statement = "fn:lower-case(%(attribute)s)" % {'attribute': attr}
-        # TRANSLATORS: _("the") should be a space-separated list of all-lowercase articles
-        # (such as 'the') that should be ignored when alphabetizing artists/albums. This
-        # list should include 'the' regardless of language. If some articles occur more
-        # frequently than others, most common should appear first, least common last.
+    def _order_by_statement(attr):
+        """Returns a specifically sorted SPARQL ORDER BY statement.
+
+        Returns a SPARQL ORDER BY statement sorting by the given
+        attribute, ignoring articles as defined in _("the") as well as
+        the punctuation at the start and the end of the string.
+
+        :param str attr: The attribute to order by
+        :return: The sparql order by statement
+        :rtype: str
+        """
+        return_statement = """fn:replace(fn:lower-case(%(attribute)s),
+        "^[%(punctuation)s]+|[%(punctuation)s]+$", "")
+        """.replace('\n', ' ').strip() % {
+            'attribute': attr,
+            'punctuation': PUNCTUATION_FILTER
+        }
+
+        # TRANSLATORS: The following translatable string should be a
+        # space-separated list of all-lowercase articles that should be
+        # ignored when alphabetizing artists/albums. This list should
+        # include the basic english translatable strings regardless of
+        # language because it is so universal.
+        # If some articles occur more frequently than others, the most
+        # common one should appear first, the least common one last.
         for article in reversed(_("the a an").split(" ")):
-            return_statement = '''IF(STRSTARTS(%(attribute)s, "%(article)s"),
-            SUBSTR(%(attribute)s, %(substr_start)s),
-            %(nested_if)s)''' % {
+            return_statement = """IF(STRSTARTS(%(attribute)s, "%(article)s"),
+            SUBSTR(%(attribute)s, %(substr_start)s), %(nested_if)s)
+            """ % {
                 'attribute': attr,
                 'article': article + " ",
                 'substr_start': str(len(article) + 2),
-                'nested_if': return_statement}
+                'nested_if': return_statement
+            }
         return return_statement
 
     @staticmethod
@@ -122,10 +140,9 @@ class Query():
     SELECT
         rdf:type(?album)
         tracker:id(?album) AS ?id
-        nmm:artistName(?albumArtist) AS ?artist
+        nmm:artistName(?albumArtist) AS ?album_artist
+        nmm:artistName(?performer) AS ?artist
         ?title
-        tracker:coalesce((SELECT GROUP_CONCAT(nmm:artistName(?albumArtist), ',') { ?album nmm:albumArtist ?albumArtist }),
-                         (SELECT GROUP_CONCAT(nmm:artistName(?performer), ',') { ?song nmm:performer ?performer })) AS ?performer
         COUNT(?song) AS ?childcount
         YEAR(MAX(nie:contentCreated(?song))) AS ?creation_date
     {
@@ -146,8 +163,8 @@ class Query():
     '''.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
             'music_dir': Query.MUSIC_URI,
-            'album_order': Query.order_by_statement("?title_lower"),
-            'artist_order': Query.order_by_statement("?artist_lower"),
+            'album_order': Query._order_by_statement("?title_lower"),
+            'artist_order': Query._order_by_statement("?artist_lower"),
         }
 
         return query
@@ -158,10 +175,9 @@ class Query():
     SELECT
         rdf:type(?album)
         tracker:id(?album) AS ?id
-        ?author
+        nmm:artistName(?performer) AS ?artist
+        nmm:artistName(?albumArtist) AS ?album_artist
         ?title
-        tracker:coalesce((SELECT GROUP_CONCAT(nmm:artistName(?albumArtist), ',') { ?album nmm:albumArtist ?albumArtist }),
-                         (SELECT GROUP_CONCAT(nmm:artistName(?performer), ',') { ?song nmm:performer ?performer })) AS ?performer
         COUNT(?song) AS ?childcount
         YEAR(MAX(nie:contentCreated(?song))) AS ?creation_date
     {
@@ -169,15 +185,12 @@ class Query():
         ?album a nmm:MusicAlbum ;
                nmm:albumArtist ?albumArtist ;
                nie:title ?title .
-        ?albumArtist nmm:artistName ?author .
         ?song nmm:musicAlbum ?album ;
               nmm:performer ?performer .
-
-        BIND(LCASE(?author) AS ?author_lower) .
+        BIND(LCASE(nmm:artistName(?albumArtist)) AS ?artist_lower) .
         BIND(LCASE(?title) AS ?title_lower) .
         BIND((%(artist_order)s) AS ?artist_collation) .
         BIND((%(album_order)s) AS ?title_collation) .
-
         FILTER(STRSTARTS(nie:url(?song), '%(music_dir)s/'))
     }
     GROUP BY ?album
@@ -185,8 +198,8 @@ class Query():
     '''.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
             'music_dir': Query.MUSIC_URI,
-            'artist_order': Query.order_by_statement("?author_lower"),
-            'album_order': Query.order_by_statement("?title_lower")
+            'artist_order': Query._order_by_statement("?artist_lower"),
+            'album_order': Query._order_by_statement("?title_lower")
         }
 
         return query
@@ -194,15 +207,15 @@ class Query():
     @staticmethod
     def songs(where_clause):
         query = '''
-    SELECT
+    SELECT DISTINCT
         rdf:type(?song)
         tracker:id (?song) AS ?id
         ?url
         nie:title(?song) AS ?title
-        nmm:artistName (nmm:performer(?song)) AS ?artist
-        nie:title (nmm:musicAlbum (?song)) AS ?album
-        nfo:duration (?song) AS ?duration
-        IF (BOUND (?tag), 'b', '') AS ?lyrics
+        nmm:artistName(nmm:performer(?song)) AS ?artist
+        nie:title(nmm:musicAlbum(?song)) AS ?album
+        nfo:duration(?song) AS ?duration
+        IF (BOUND(?tag), 'b', '') AS ?lyrics
     {
         %(where_clause)s
         ?song a nmm:MusicPiece ;
@@ -214,7 +227,7 @@ class Query():
             nie:title ?title .
         OPTIONAL { ?song nao:hasTag ?tag .
                    FILTER (?tag = nao:predefined-tag-favorite) } .
-        FILTER (STRSTARTS (?url, '%(music_dir)s/'))
+        FILTER(STRSTARTS(?url, '%(music_dir)s/'))
     }
     ORDER BY ?artist ?album nmm:setNumber(?disc) nmm:trackNumber(?song)
     '''.replace('\n', ' ').strip() % {
@@ -234,14 +247,13 @@ class Query():
         nfo:entryCounter(?playlist) AS ?childcount
         {
             %(where_clause)s
-            ?playlist a nmm:Playlist .
-            OPTIONAL {
-                ?playlist nie:url ?url ;
-                          tracker:available true .
-            FILTER (STRSTARTS (?url, '%(music_dir)s/'))
-            }
+            OPTIONAL { ?playlist nie:url ?url;
+                       tracker:available ?available . }
+            FILTER ( (STRSTARTS(?url, '%(music_dir)s/') && ?available)
+                      || !BOUND(nfo:belongsToContainer(?playlist)) )
+            OPTIONAL { ?playlist nao:hasTag ?tag }
         }
-    ORDER BY LCASE(?title)
+    ORDER BY !BOUND(?tag) LCASE(?title)
     '''.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
             'music_dir': Query.MUSIC_URI
@@ -336,12 +348,12 @@ class Query():
         tracker:id(?album) AS ?id
         (
             SELECT
-                nmm:artistName(?artist)
+                nmm:artistName(?album_artist)
             WHERE {
-                ?album nmm:albumArtist ?artist
+                ?album nmm:albumArtist ?album_artist
             }
             LIMIT 1
-        ) AS ?artist
+        ) AS ?album_artist
         nie:title(?album) AS ?title
         nie:title(?album) AS ?album
     WHERE {
@@ -364,12 +376,12 @@ class Query():
         tracker:id(?album) AS ?id
         (
             SELECT
-                nmm:artistName(?artist)
+                nmm:artistName(?album_artist)
             WHERE {
-                ?album nmm:albumArtist ?artist
+                ?album nmm:albumArtist ?album_artist
             }
             LIMIT 1
-        ) AS ?artist
+        ) AS ?album_artist
         nie:title(?album) AS ?title
         nie:title(?album) AS ?album
     WHERE {
