@@ -47,7 +47,8 @@ from gnomemusic.toolbar import ToolbarState
 import gnomemusic.widgets as Widgets
 from gnomemusic.player import DiscoveryStatus
 from gnomemusic.playlists import Playlists, StaticPlaylists
-from gnomemusic.albumArtCache import AlbumArtCache as albumArtCache
+import gnomemusic.utils as utils
+from gnomemusic.albumartcache import AlbumArtCache, DefaultIcon
 from gnomemusic import log
 import logging
 logger = logging.getLogger(__name__)
@@ -59,6 +60,8 @@ class ViewContainer(Gtk.Stack):
     nowPlayingIconName = 'media-playback-start-symbolic'
     errorIconName = 'dialog-error-symbolic'
 
+    selection_mode = GObject.Property(type=bool, default=False)
+
     def __repr__(self):
         return '<ViewContainer>'
 
@@ -88,12 +91,12 @@ class ViewContainer(Gtk.Stack):
             GObject.TYPE_BOOLEAN,
             GObject.TYPE_INT
         )
-        self.view = Gd.MainView(
-            shadow_type=Gtk.ShadowType.NONE
-        )
-        self.view.set_view_type(view_type)
+
         self._box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-        self._box.pack_start(self.view, True, True, 0)
+
+        # Setup the main view
+        self._setup_view(view_type)
+
         if use_sidebar:
             self.stack = Gtk.Stack(
                 transition_type=Gtk.StackTransitionType.SLIDE_RIGHT,
@@ -110,9 +113,6 @@ class ViewContainer(Gtk.Stack):
             self._grid.add(self._box)
 
         self.star_handler = Widgets.StarHandler(self, 9)
-        self.view.click_handler = self.view.connect('item-activated', self._on_item_activated)
-        # self.star_handler.star_renderer_click = False
-        self.view.connect('selection-mode-request', self._on_selection_mode_request)
         self._cursor = None
         self.window = window
         self.header_bar = window.toolbar
@@ -129,8 +129,10 @@ class ViewContainer(Gtk.Stack):
         self.show_all()
         self.view.hide()
         self._items = []
-        self.cache = albumArtCache.get_default()
-        self._loadingIcon = self.cache.get_default_icon(self._iconWidth, self._iconHeight, True)
+        self.cache = AlbumArtCache.get_default()
+        self._loading_icon = DefaultIcon().get(self._iconWidth,
+                                               self._iconHeight,
+                                               DefaultIcon.Type.loading)
 
         self._init = False
         grilo.connect('ready', self._on_grilo_ready)
@@ -145,20 +147,36 @@ class ViewContainer(Gtk.Stack):
     def _on_changes_pending(self, data=None):
         pass
 
+    @log
+    def _setup_view(self, view_type):
+        self.view = Gd.MainView(shadow_type=Gtk.ShadowType.NONE)
+        self.view.set_view_type(view_type)
+
+        self.view.click_handler = self.view.connect('item-activated', self._on_item_activated)
+        self.view.connect('selection-mode-request', self._on_selection_mode_request)
+
+        self.view.bind_property('selection-mode', self, 'selection_mode',
+                                GObject.BindingFlags.BIDIRECTIONAL)
+
+        self.view.connect('view-selection-changed', self._on_view_selection_changed)
+
+        self._box.pack_start(self.view, True, True, 0)
+
     @log
     def _on_header_bar_toggled(self, button):
-        if button.get_active():
-            self.view.set_selection_mode(True)
+        self.selection_mode = button.get_active()
+
+        if self.selection_mode:
             self.header_bar.set_selection_mode(True)
             self.player.actionbar.set_visible(False)
             self.selection_toolbar.actionbar.set_visible(True)
             self.selection_toolbar._add_to_playlist_button.set_sensitive(False)
             self.selection_toolbar._remove_from_playlist_button.set_sensitive(False)
         else:
-            self.view.set_selection_mode(False)
             self.header_bar.set_selection_mode(False)
             self.player.actionbar.set_visible(self.player.currentTrack is not None)
             self.selection_toolbar.actionbar.set_visible(False)
+            self.unselect_all()
 
     @log
     def _on_cancel_button_clicked(self, button):
@@ -179,14 +197,22 @@ class ViewContainer(Gtk.Stack):
 
     @log
     def _on_view_selection_changed(self, widget):
+
+        if not self.selection_mode:
+            return
+
         items = self.view.get_selection()
+        self.update_header_from_selection(len(items))
+
+    @log
+    def update_header_from_selection(self, n_items):
         self.selection_toolbar._add_to_playlist_button.\
-            set_sensitive(len(items) > 0)
+            set_sensitive(n_items > 0)
         self.selection_toolbar._remove_from_playlist_button.\
-            set_sensitive(len(items) > 0)
-        if len(items) > 0:
+            set_sensitive(n_items > 0)
+        if n_items > 0:
             self.header_bar._selection_menu_label.set_text(
-                ngettext("Selected %d item", "Selected %d items", len(items)) % len(items))
+                ngettext("Selected %d item", "Selected %d items", n_items) % n_items)
         else:
             self.header_bar._selection_menu_label.set_text(_("Click on items to select them"))
 
@@ -214,17 +240,15 @@ class ViewContainer(Gtk.Stack):
             return
 
         self._offset += 1
-        artist = item.get_string(Grl.METADATA_KEY_ARTIST)\
-            or item.get_author()\
-            or _("Unknown Artist")
-        title = albumArtCache.get_media_title(item)
+        artist = utils.get_artist_name(item)
+        title = AlbumArtCache.get_media_title(item)
         # item.set_title(title)
 
         _iter = self.model.append(None)
         self.model.set(_iter,
                        [0, 1, 2, 3, 4, 5, 7, 9],
                        [str(item.get_id()), '', title,
-                        artist, self._loadingIcon, item,
+                        artist, self._loading_icon, item,
                         0, False])
         self.cache.lookup(item, self._iconWidth, self._iconHeight, self._on_lookup_ready,
                           _iter, artist, title)
@@ -253,6 +277,39 @@ class ViewContainer(Gtk.Stack):
     def _on_list_widget_star_render(self, col, cell, model, _iter, data):
         pass
 
+    @log
+    def _set_selection(self, value, parent=None):
+        count = 0
+        _iter = self.model.iter_children(parent)
+        while _iter is not None:
+            if self.model.iter_has_child(_iter):
+                count += self._set_selection(value, _iter)
+            if self.model[_iter][5]:
+                self.model[_iter][6] = value
+                count += 1
+            _iter = self.model.iter_next(_iter)
+        return count
+
+    @log
+    def select_all(self):
+        """Select all the available tracks."""
+        count = self._set_selection(True)
+
+        if count > 0:
+            self.selection_toolbar._add_to_playlist_button.set_sensitive(True)
+            self.selection_toolbar._remove_from_playlist_button.set_sensitive(True)
+
+        self.update_header_from_selection(count)
+        self.view.queue_draw()
+
+    @log
+    def unselect_all(self):
+        """Unselects all the selected tracks."""
+        self._set_selection(False)
+        self.selection_toolbar._add_to_playlist_button.set_sensitive(False)
+        self.selection_toolbar._remove_from_playlist_button.set_sensitive(False)
+        self.header_bar._selection_menu_label.set_text(_("Click on items to select them"))
+        self.queue_draw()
 
 # Class for the Empty View
 class Empty(Gtk.Stack):
@@ -308,18 +365,19 @@ class Albums(ViewContainer):
 
     @log
     def __init__(self, window, player):
-        ViewContainer.__init__(self, 'albums', _("Albums"), window, Gd.MainViewType.ICON)
+        ViewContainer.__init__(self, 'albums', _("Albums"), window, None)
         self._albumWidget = Widgets.AlbumWidget(player, self)
         self.player = player
         self.add(self._albumWidget)
         self.albums_selected = []
+        self.all_items = []
         self.items_selected = []
         self.items_selected_callback = None
         self._add_list_renderers()
 
     @log
     def _on_changes_pending(self, data=None):
-        if (self._init and self.header_bar._selectionMode is False):
+        if (self._init and not self.header_bar._selectionMode):
             self._offset = 0
             self._init = True
             GLib.idle_add(self.populate)
@@ -327,31 +385,56 @@ class Albums(ViewContainer):
 
     @log
     def _on_selection_mode_changed(self, widget, data=None):
-        if self.header_bar._selectionMode is False and grilo.changes_pending['Albums'] is True:
+        if (not self.header_bar._selectionMode
+                and grilo.changes_pending['Albums']):
             self._on_changes_pending()
 
+    @log
+    def _setup_view(self, view_type):
+        self.view = Gtk.FlowBox(homogeneous=True,
+                                hexpand=True,
+                                halign=Gtk.Align.FILL,
+                                selection_mode=Gtk.SelectionMode.NONE,
+                                margin=18,
+                                row_spacing=12,
+                                column_spacing=6,
+                                min_children_per_line=1,
+                                max_children_per_line=25)
+
+        self.view.connect('child-activated', self._on_child_activated)
+
+        scrolledwin = Gtk.ScrolledWindow()
+        scrolledwin.add(self.view)
+        scrolledwin.show()
+
+        self._box.add(scrolledwin)
+
     @log
     def _back_button_clicked(self, widget, data=None):
         self.header_bar.reset_header_title()
         self.set_visible_child(self._grid)
 
     @log
-    def _on_item_activated(self, widget, id, path):
+    def _on_child_activated(self, widget, child, user_data=None):
+        item = child.media_item
+
         if self.star_handler.star_renderer_click:
             self.star_handler.star_renderer_click = False
             return
 
-        try:
-            _iter = self.model.get_iter(path)
-        except TypeError:
+        # Toggle the selection when in selection mode
+        if self.selection_mode:
+            child.check.set_active(not child.check.get_active())
             return
-        title = self.model.get_value(_iter, 2)
-        self._artist = self.model.get_value(_iter, 3)
-        item = self.model.get_value(_iter, 5)
+
+        title = AlbumArtCache.get_media_title(item)
+        self._escaped_title = title
+        self._artist = utils.get_artist_name(item)
+
         self._albumWidget.update(self._artist, title, item,
                                  self.header_bar, self.selection_toolbar)
+
         self.header_bar.set_state(ToolbarState.CHILD_VIEW)
-        self._escaped_title = albumArtCache.get_media_title(item)
         self.header_bar.header_bar.set_title(self._escaped_title)
         self.header_bar.header_bar.sub_title = self._artist
         self.set_visible_child(self._albumWidget)
@@ -379,11 +462,85 @@ class Albums(ViewContainer):
             self.items_selected = []
             self.items_selected_callback = callback
             self.albums_index = 0
-            self.albums_selected = [self.model.get_value(self.model.get_iter(path), 5)
-                                    for path in self.view.get_selection()]
             if len(self.albums_selected):
                 self._get_selected_album_songs()
 
+    @log
+    def _add_item(self, source, param, item, remaining=0, data=None):
+        self.window.notification.set_timeout(0)
+
+        if item:
+            # Store all items to optimize 'Select All' action
+            self.all_items.append(item)
+
+            # Add to the flowbox
+            child = self._create_album_item(item)
+            self.view.add(child)
+        elif remaining == 0:
+                self.window.notification.dismiss()
+                self.view.show()
+
+    def _create_album_item(self, item):
+        artist = utils.get_artist_name(item)
+        title = AlbumArtCache.get_media_title(item)
+
+        builder = Gtk.Builder.new_from_resource('/org/gnome/Music/AlbumCover.ui')
+
+        child = Gtk.FlowBoxChild()
+        child.image = builder.get_object('image')
+        child.check = builder.get_object('check')
+        child.title = builder.get_object('title')
+        child.subtitle = builder.get_object('subtitle')
+        child.events = builder.get_object('events')
+        child.media_item = item
+
+        child.title.set_label(title)
+        child.subtitle.set_label(artist)
+        child.image.set_from_pixbuf(self._loading_icon)
+        # In the case of off-sized icons (eg. provided in the soundfile)
+        # keep the size request equal to all other icons to get proper
+        # alignment with GtkFlowBox.
+        child.image.set_property("width-request", self._iconWidth)
+        child.image.set_property("height-request", self._iconHeight)
+
+        child.events.connect('button-release-event',
+                             self._on_album_event_triggered,
+                             child)
+
+        child.check_handler_id = child.check.connect('notify::active',
+                                                     self._on_child_toggled,
+                                                     child)
+
+        child.check.bind_property('visible', self, 'selection_mode',
+                                  GObject.BindingFlags.BIDIRECTIONAL)
+
+        child.add(builder.get_object('main_box'))
+        child.show()
+
+        self.cache.lookup(item, self._iconWidth, self._iconHeight, self._on_lookup_ready,
+                          child, artist, title)
+
+        return child
+
+    @log
+    def _on_album_event_triggered(self, evbox, event, child):
+        if event.button is 3:
+            self._on_selection_mode_request()
+            if self.selection_mode:
+                child.check.set_active(True)
+
+    def _on_lookup_ready(self, icon, path, child):
+        child.image.set_from_pixbuf(icon)
+
+    @log
+    def _on_child_toggled(self, check, pspec, child):
+        if check.get_active() and not child.media_item in self.albums_selected:
+            self.albums_selected.append(child.media_item)
+        elif not check.get_active() and child.media_item in self.albums_selected:
+            self.albums_selected.remove(child.media_item)
+
+        self.update_header_from_selection(len(self.albums_selected))
+
     @log
     def _get_selected_album_songs(self):
         grilo.populate_album_songs(
@@ -401,6 +558,31 @@ class Albums(ViewContainer):
             else:
                 self.items_selected_callback(self.items_selected)
 
+    def _toggle_all_selection(self, selected):
+        """
+        Selects or unselects all items without sending the notify::active
+        signal for performance purposes.
+        """
+        for child in self.view.get_children():
+            GObject.signal_handler_block(child.check, child.check_handler_id)
+
+            # Set the checkbutton state without emiting the signal
+            child.check.set_active(selected)
+
+            GObject.signal_handler_unblock(child.check, child.check_handler_id)
+
+        self.update_header_from_selection(len(self.albums_selected))
+
+    @log
+    def select_all(self):
+        self.albums_selected = list(self.all_items)
+        self._toggle_all_selection(True)
+
+    @log
+    def unselect_all(self):
+        self.albums_selected = []
+        self._toggle_all_selection(False)
+
 
 class Songs(ViewContainer):
 
@@ -473,16 +655,14 @@ class Songs(ViewContainer):
                 self.view.show()
             return
         self._offset += 1
-        item.set_title(albumArtCache.get_media_title(item))
-        artist = item.get_string(Grl.METADATA_KEY_ARTIST)\
-            or item.get_author()\
-            or _("Unknown Artist")
+        item.set_title(AlbumArtCache.get_media_title(item))
+        artist = utils.get_artist_name(item)
         if item.get_url() is None:
             return
         self.model.insert_with_valuesv(
             -1,
             [2, 3, 5, 9],
-            [albumArtCache.get_media_title(item),
+            [AlbumArtCache.get_media_title(item),
              artist, item, bool(item.get_lyrics())])
         # TODO: change "bool(item.get_lyrics())" --> item.get_favourite() once query works properly
 
@@ -517,7 +697,7 @@ class Songs(ViewContainer):
                                  self._on_list_widget_title_render, None)
         cols[0].add_attribute(title_renderer, 'text', 2)
 
-        self.star_handler._add_star_renderers(list_widget, cols)
+        self.star_handler.add_star_renderers(list_widget, cols)
 
         duration_renderer = Gd.StyledTextRenderer(
             xpad=32,
@@ -574,7 +754,7 @@ class Songs(ViewContainer):
     def _on_list_widget_type_render(self, coll, cell, model, _iter, data):
         item = model.get_value(_iter, 5)
         if item:
-            cell.set_property('text', item.get_string(Grl.METADATA_KEY_ALBUM) or _("Unknown Album"))
+            cell.set_property('text', item.get_album() or _("Unknown Album"))
 
     def _on_list_widget_icon_render(self, col, cell, model, _iter, data):
         if not self.player.currentTrackUri:
@@ -728,9 +908,7 @@ class Artists (ViewContainer):
                 self.view.show()
             return
         self._offset += 1
-        artist = item.get_string(Grl.METADATA_KEY_ARTIST)\
-            or item.get_author()\
-            or _("Unknown Artist")
+        artist = utils.get_artist_name(item)
         if not artist.casefold() in self._artists:
             _iter = self.model.insert_with_valuesv(-1, [2], [artist])
             self._artists[artist.casefold()] = {'iter': _iter, 'albums': [], 'widget': None}
@@ -870,6 +1048,7 @@ class Playlist(ViewContainer):
         self.current_playlist = None
         self.current_playlist_index = None
         self.pl_todelete = None
+        self.pl_todelete_index = None
         self.really_delete = True
         self.songs_count = 0
         self.window = window
@@ -915,7 +1094,7 @@ class Playlist(ViewContainer):
                                  self._on_list_widget_title_render, None)
         cols[0].add_attribute(title_renderer, 'text', 2)
 
-        self.star_handler._add_star_renderers(list_widget, cols)
+        self.star_handler.add_star_renderers(list_widget, cols)
 
         duration_renderer = Gd.StyledTextRenderer(
             xpad=32,
@@ -987,7 +1166,7 @@ class Playlist(ViewContainer):
 
         item = model.get_value(_iter, 5)
         if item:
-            cell.set_property('text', item.get_string(Grl.METADATA_KEY_ALBUM) or _("Unknown Album"))
+            cell.set_property('text', item.get_album() or _("Unknown Album"))
 
     def _on_list_widget_icon_render(self, col, cell, model, _iter, data):
         if not self.player.currentTrackUri:
@@ -1042,7 +1221,7 @@ class Playlist(ViewContainer):
         _iter = self.playlists_model.insert_with_valuesv(
             index,
             [2, 5],
-            [albumArtCache.get_media_title(item), item])
+            [AlbumArtCache.get_media_title(item), item])
         if self.playlists_model.iter_n_children(None) == 1:
             _iter = self.playlists_model.get_iter_first()
             selection = self.playlists_sidebar.get_generic_view().get_selection()
@@ -1158,11 +1337,9 @@ class Playlist(ViewContainer):
             self.emit('playlist-songs-loaded')
             return
         self._offset += 1
-        title = albumArtCache.get_media_title(item)
+        title = AlbumArtCache.get_media_title(item)
         item.set_title(title)
-        artist = item.get_string(Grl.METADATA_KEY_ARTIST)\
-            or item.get_author()\
-            or _("Unknown Artist")
+        artist = item.get_artist() or _("Unknown Artist")
         model.insert_with_valuesv(
             -1,
             [2, 3, 5, 9],
@@ -1202,26 +1379,26 @@ class Playlist(ViewContainer):
     @log
     def stage_playlist_for_deletion(self):
         self.model.clear()
+        self.pl_todelete_index = self.current_playlist_index
         _iter = self.playlists_sidebar.get_generic_view().get_selection().get_selected()[1]
-
+        self.pl_todelete = self.playlists_model.get_value(_iter, 5)
+      
         if not _iter:
             return
 
         iter_next = self.playlists_model.iter_next(_iter)\
             or self.playlists_model.iter_previous(_iter)
+        self.playlists_model.remove(_iter)
+
         if iter_next:
             selection = self.playlists_sidebar.get_generic_view().get_selection()
             selection.select_iter(iter_next)
             self.playlists_sidebar.emit('item-activated', '0',
                                         self.playlists_model.get_path(iter_next))
 
-        playlist = self.playlists_model.get_value(_iter, 5)
-        self.pl_todelete = playlist
-        self.playlists_model.remove(_iter)
-
     @log
-    def undo_playlist_deletion(self, deletion_index):
-        self._add_playlist_item_to_model(self.pl_todelete, index=deletion_index)
+    def undo_playlist_deletion(self):
+        self._add_playlist_item_to_model(self.pl_todelete, self.pl_todelete_index)
 
     @log
     def _on_delete_activate(self, menuitem, data=None):
@@ -1347,8 +1524,12 @@ class Search(ViewContainer):
         self.iter_to_clean = None
         self._iconHeight = 48
         self._iconWidth = 48
-        self._loadingIcon = self.cache.get_default_icon(self._iconWidth, self._iconHeight, True)
-        self._noAlbumArtIcon = self.cache.get_default_icon(self._iconWidth, self._iconHeight, False)
+        self._loading_icon = DefaultIcon().get(self._iconWidth,
+                                               self._iconHeight,
+                                               DefaultIcon.Type.loading)
+        self._no_albumart_icon = DefaultIcon().get(self._iconWidth,
+                                                   self._iconHeight,
+                                                   DefaultIcon.Type.music)
         self._add_list_renderers()
         self.player = player
         self.head_iters = [None, None, None, None]
@@ -1406,7 +1587,7 @@ class Search(ViewContainer):
             self._albumWidget.update(artist, title, item,
                                      self.header_bar, self.selection_toolbar)
             self.header_bar.set_state(ToolbarState.SEARCH_VIEW)
-            escaped_title = albumArtCache.get_media_title(item)
+            escaped_title = AlbumArtCache.get_media_title(item)
             self.header_bar.header_bar.set_title(escaped_title)
             self.header_bar.header_bar.sub_title = artist
             self.set_visible_child(self._albumWidget)
@@ -1452,17 +1633,14 @@ class Search(ViewContainer):
         if data != self.model:
             return
 
-        artist = item.get_string(Grl.METADATA_KEY_ARTIST) \
-            or item.get_author() \
-            or _("Unknown Artist")
-        album = item.get_string(Grl.METADATA_KEY_ALBUM) \
-            or _("Unknown Album")
+        artist = utils.get_artist_name(item)
+        album = item.get_album() or _("Unknown Album")
 
         key = '%s-%s' % (artist, album)
         if key not in self._albums:
             self._albums[key] = Grl.Media()
             self._albums[key].set_title(album)
-            self._albums[key].add_author(artist)
+            self._albums[key].add_artist(artist)
             self._albums[key].set_source(source.get_id())
             self._albums[key].tracks = []
             self._add_item(source, None, self._albums[key], 0, [self.model, 'album'])
@@ -1502,11 +1680,9 @@ class Search(ViewContainer):
             return
 
         self._offset += 1
-        title = albumArtCache.get_media_title(item)
+        title = AlbumArtCache.get_media_title(item)
         item.set_title(title)
-        artist = item.get_string(Grl.METADATA_KEY_ARTIST) \
-            or item.get_author() \
-            or _("Unknown Artist")
+        artist = utils.get_artist_name(item)
 
         group = 3
         try:
@@ -1520,7 +1696,7 @@ class Search(ViewContainer):
                 self.head_iters[group], -1,
                 [0, 2, 3, 4, 5, 9, 11],
                 [str(item.get_id()), title, artist,
-                 self._loadingIcon, item, 2, category])
+                 self._loading_icon, item, 2, category])
             self.cache.lookup(item, self._iconWidth, self._iconHeight, self._on_lookup_ready,
                               _iter, artist, title)
         elif category == 'song':
@@ -1528,14 +1704,14 @@ class Search(ViewContainer):
                 self.head_iters[group], -1,
                 [0, 2, 3, 4, 5, 9, 11],
                 [str(item.get_id()), title, artist,
-                 self._noAlbumArtIcon, item, 2 if source.get_id() != 'grl-tracker-source' else bool(item.get_lyrics()), category])
+                 self._no_albumart_icon, item, 2 if source.get_id() != 'grl-tracker-source' else bool(item.get_lyrics()), category])
         else:
             if not artist.casefold() in self._artists:
                 _iter = self.model.insert_with_values(
                     self.head_iters[group], -1,
                     [0, 2, 4, 5, 9, 11],
                     [str(item.get_id()), artist,
-                     self._loadingIcon, item, 2, category])
+                     self._loading_icon, item, 2, category])
                 self.cache.lookup(item, self._iconWidth, self._iconHeight, self._on_lookup_ready,
                                   _iter, artist, title)
                 self._artists[artist.casefold()] = {'iter': _iter, 'albums': []}
@@ -1566,7 +1742,7 @@ class Search(ViewContainer):
                                  self._on_list_widget_title_render, None)
         cols[0].add_attribute(title_renderer, 'text', 2)
 
-        self.star_handler._add_star_renderers(list_widget, cols, hidden=False)
+        self.star_handler.add_star_renderers(list_widget, cols, hidden=False)
 
         cells = cols[0].get_cells()
         cols[0].reorder(cells[0], -1)
