@@ -36,7 +36,7 @@ from gettext import gettext as _, ngettext
 
 from gnomemusic import TrackerWrapper
 from gnomemusic.toolbar import Toolbar, ToolbarState
-from gnomemusic.player import Player, SelectionToolbar
+from gnomemusic.player import Player, SelectionToolbar, RepeatType
 from gnomemusic.query import Query
 import gnomemusic.view as Views
 import gnomemusic.widgets as Widgets
@@ -76,6 +76,7 @@ class Window(Gtk.ApplicationWindow):
 
         self.prev_view = None
         self.curr_view = None
+        self.pl_todelete_notification = None
 
         size_setting = self.settings.get_value('window-size')
         if isinstance(size_setting[0], int) and isinstance(size_setting[1], int):
@@ -192,6 +193,11 @@ class Window(Gtk.ApplicationWindow):
             transition_duration=100,
             visible=True,
             can_focus=False)
+
+        # Add the 'background' styleclass so it properly hides the
+        # bottom line of the searchbar
+        self._stack.get_style_context().add_class('background')
+
         self._overlay = Gtk.Overlay(child=self._stack)
         self._overlay.add_overlay(self.toolbar.dropdown)
         self.set_titlebar(self.toolbar.header_bar)
@@ -277,48 +283,25 @@ class Window(Gtk.ApplicationWindow):
 
         self.views[0].populate()
 
-    @log
-    def _set_selection(self, model, value, parent=None):
-        count = 0
-        _iter = model.iter_children(parent)
-        while _iter is not None:
-            if model.iter_has_child(_iter):
-                count += self._set_selection(model, value, _iter)
-            if model[_iter][5]:
-                model.set(_iter, [6], [value])
-                count += 1
-            _iter = model.iter_next(_iter)
-        return count
-
     @log
     def _on_select_all(self, action, param):
         if self.toolbar._selectionMode is False:
             return
         if self.toolbar._state == ToolbarState.MAIN:
-            model = self._stack.get_visible_child().model
+            view = self._stack.get_visible_child()
         else:
-            model = self._stack.get_visible_child().get_visible_child().model
-        count = self._set_selection(model, True)
-        if count > 0:
-            self.toolbar._selection_menu_label.set_text(
-                ngettext("Selected %d item", "Selected %d items", count) % count)
-            self.selection_toolbar._add_to_playlist_button.set_sensitive(True)
-            self.selection_toolbar._remove_from_playlist_button.set_sensitive(True)
-        elif count == 0:
-            self.toolbar._selection_menu_label.set_text(_("Click on items to select them"))
-        self._stack.get_visible_child().queue_draw()
+            view = self._stack.get_visible_child().get_visible_child()
+
+        view.select_all()
 
     @log
     def _on_select_none(self, action, param):
         if self.toolbar._state == ToolbarState.MAIN:
-            model = self._stack.get_visible_child().model
+            view = self._stack.get_visible_child()
         else:
-            model = self._stack.get_visible_child().get_visible_child().model
-        self._set_selection(model, False)
-        self.selection_toolbar._add_to_playlist_button.set_sensitive(False)
-        self.selection_toolbar._remove_from_playlist_button.set_sensitive(False)
-        self.toolbar._selection_menu_label.set_text(_("Click on items to select them"))
-        self._stack.get_visible_child().queue_draw()
+            view = self._stack.get_visible_child().get_visible_child()
+
+        view.unselect_all()
 
     def _show_notification(self):
         self.notification_handler = None
@@ -345,6 +328,11 @@ class Window(Gtk.ApplicationWindow):
 
     @log
     def _init_playlist_removal_notification(self):
+        if self.pl_todelete_notification:
+            self.views[3].really_delete = False
+            self.pl_todelete_notification.destroy()
+            Views.playlists.delete_playlist(self.views[3].pl_todelete)
+
         self.notification = Gd.Notification()
         self.notification.set_timeout(20)
 
@@ -360,14 +348,14 @@ class Window(Gtk.ApplicationWindow):
 
         self.notification.show_all()
         self._overlay.add_overlay(self.notification)
-
-        self.notification.deletion_index = self.views[3].current_playlist_index
+        self.pl_todelete_notification = self.notification
 
         self.notification.connect("dismissed", self._playlist_removal_notification_dismissed)
         undo_button.connect("clicked", self._undo_deletion)
 
     @log
     def _playlist_removal_notification_dismissed(self, widget):
+        self.pl_todelete_notification = None
         if self.views[3].really_delete:
             Views.playlists.delete_playlist(self.views[3].pl_todelete)
         else:
@@ -377,7 +365,7 @@ class Window(Gtk.ApplicationWindow):
     def _undo_deletion(self, widget):
         self.views[3].really_delete = False
         self.notification.dismiss()
-        self.views[3].undo_playlist_deletion(self.notification.deletion_index)
+        self.views[3].undo_playlist_deletion()
 
     @log
     def _on_key_press(self, widget, event):
@@ -389,12 +377,55 @@ class Window(Gtk.ApplicationWindow):
             if (event.keyval == Gdk.KEY_f and
                     event_and_modifiers == Gdk.ModifierType.CONTROL_MASK):
                 self.toolbar.searchbar.toggle_bar()
+            # Play / Pause on Ctrl + P
+            if (event.keyval == Gdk.KEY_p
+                    or event.keyval == Gdk.KEY_space
+                    and event_and_modifiers == Gdk.ModifierType.CONTROL_MASK):
+                 self.player.play_pause()
+            # Play previous on Ctrl + B
+            if (event.keyval == Gdk.KEY_b
+                    and event_and_modifiers == Gdk.ModifierType.CONTROL_MASK):
+                self.player.play_previous()
+            # Play next on Ctrl + N
+            if (event.keyval == Gdk.KEY_n
+                    and event_and_modifiers == Gdk.ModifierType.CONTROL_MASK):
+                self.player.play_next()
+            # Toggle repeat on Ctrl + R
+            if (event.keyval == Gdk.KEY_r
+                    and event_and_modifiers == Gdk.ModifierType.CONTROL_MASK):
+                if self.player.get_repeat_mode() == RepeatType.SONG:
+                    self.player.set_repeat_mode(RepeatType.NONE)
+                else:
+                    self.player.set_repeat_mode(RepeatType.SONG)
+            # Toggle shuffle on Ctrl + S
+            if (event.keyval == Gdk.KEY_s
+                    and event_and_modifiers == Gdk.ModifierType.CONTROL_MASK):
+                if self.player.get_repeat_mode() == RepeatType.SHUFFLE:
+                    self.player.set_repeat_mode(RepeatType.NONE)
+                else:
+                    self.player.set_repeat_mode(RepeatType.SHUFFLE)
             # Go back from Album view on Alt + Left
             if (event.keyval == Gdk.KEY_Left and
                     event_and_modifiers == Gdk.ModifierType.MOD1_MASK):
                 if (self.toolbar._state != ToolbarState.MAIN):
                     self.curr_view.set_visible_child(self.curr_view._grid)
                     self.toolbar.set_state(ToolbarState.MAIN)
+            # Go to Albums view on Ctrl + 1
+            if (event.keyval == Gdk.KEY_1
+                    and event_and_modifiers == Gdk.ModifierType.CONTROL_MASK):
+                self._toggle_view(0, 0)
+            # Go to Artists view on Ctrl + 2
+            if (event.keyval == Gdk.KEY_2
+                    and event_and_modifiers == Gdk.ModifierType.CONTROL_MASK):
+                self._toggle_view(0, 1)
+            # Go to Songs view on Ctrl + 3
+            if (event.keyval == Gdk.KEY_3
+                    and event_and_modifiers == Gdk.ModifierType.CONTROL_MASK):
+                self._toggle_view(0, 2)
+            # Go to Playlists view on Ctrl + 4
+            if (event.keyval == Gdk.KEY_4
+                    and event_and_modifiers == Gdk.ModifierType.CONTROL_MASK):
+                self._toggle_view(0, 3)
         else:
             if (event.keyval == Gdk.KEY_Delete):
                 if self._stack.get_visible_child() == self.views[3]:
@@ -409,19 +440,21 @@ class Window(Gtk.ApplicationWindow):
         # Open search bar when typing printable chars if it not opened
         # Make sure we skip unprintable chars and don't grab space press
         # (this is used for play/pause)
-        if not self.toolbar.searchbar.get_reveal_child() and not event.keyval == Gdk.KEY_space:
+        if (not self.toolbar.searchbar.get_search_mode()
+                and not event.keyval == Gdk.KEY_space):
             if (event_and_modifiers == Gdk.ModifierType.SHIFT_MASK or
                     event_and_modifiers == 0) and \
                     GLib.unichar_isprint(chr(Gdk.keyval_to_unicode(event.keyval))):
                 self.toolbar.searchbar.show_bar(True)
         else:
-            if not self.toolbar.searchbar.get_reveal_child():
+            if not self.toolbar.searchbar.get_search_mode():
                 if event.keyval == Gdk.KEY_space and self.player.actionbar.get_visible():
                     if self.get_focus() != self.player.playBtn:
                         self.player.play_pause()
 
     @log
     def _notify_mode_disconnect(self, data=None):
+        self.player.Stop()
         self._stack.disconnect(self._on_notify_model_id)
 
     @log
@@ -461,7 +494,8 @@ class Window(Gtk.ApplicationWindow):
                 # We should get back to the view before the search
                 self._stack.set_visible_child(self.views[4].previous_view)
             elif (self.views[4].previous_view == self.views[0] and
-                 self.curr_view.get_visible_child() != self.curr_view._albumWidget):
+                 self.curr_view.get_visible_child() != self.curr_view._albumWidget and
+                 self.curr_view.get_visible_child() != self.curr_view._artistAlbumsWidget):
                 self._stack.set_visible_child(self.views[0])
 
             if self.toolbar._selectionMode:
